# HealthRegen release 4 by David Bowland
# ./addons/eventscripts/healthregen/healthregen.py

# >>> To configure this addon please see healthregen.cfg <<<

"""
Provides steady health regeneration for all players. Requires only ES 2.0+
"""


import es
import gamethread
import os.path


info = es.AddonInfo()
info.name = 'HealthRegen'
info.version = '4'
info.url = 'http://addons.eventscripts.com/addons/view/healthregen'
info.basename = 'healthregen'
info.author = 'SuperDave'


list_players = []

dict_options = {'healthregen_threshold':[75, 'Maximum health that can be achieved with regeneration'], 'healthregen_start_delay':[5, 'Number of seconds a player must remain unhurt to start regeneration--set to 0 for continuous regeneration'], 'healthregen_start_health':[40, 'Minimum amount of health players will have after the first regeneration iteration--set to 0 for no minimum'], 'healthregen_iteration_delay':[1, 'Seconds between regeneration iterations'], 'healthregen_iteration_amount':[2, 'Amount of health to regenerate each iteration']}


def load():
   """
   Checks to make sure critical server variables are created by the config
   """

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('healthregen') + '/healthregen.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/healthregen/healthregen.cfg')
   else:
      es.dbgmsg(0, 'HealthRegen: Unable to load healthregen.cfg! Please ensure it is in the ./healthregen/ directory.')


def es_map_start(event_var):
   """
   Cancels outstanding regen delays
   Clears all players from the player list
   """
   global list_players

   for str_userid in list_players:
      gamethread.cancelDelayed('healthregen_%s' % str_userid)

   list_players[:] = []


def player_spawn(event_var):
   """Removes the player's regeneration loop"""
   global list_players

   str_userid = event_var['userid']
   if str_userid in list_players:
      gamethread.cancelDelayed('healthregen_%s' % str_userid)
      list_players.remove(str_userid)


def player_hurt(event_var):
   """Starts a new regeneration loop if a player falls below the regeneration threshold and does not already have a regeneration loop"""
   global list_players

   str_userid = event_var['userid']
   if es.ServerVar('healthregen_threshold') > int(event_var['es_userhealth']):
      int_start_delay = int(es.ServerVar('healthregen_start_delay'))
      if int_start_delay:

         if str_userid in list_players:
            gamethread.cancelDelayed('healthregen_%s' % str_userid)
         else:
            list_players.append(str_userid)
         gamethread.delayedname(int_start_delay, 'healthregen_%s' % str_userid, execute_regen, str_userid)

      elif str_userid not in list_players:
         list_players.append(str_userid)
         gamethread.delayedname(es.ServerVar('healthregen_iteration_delay'), 'healthregen_%s' % str_userid, execute_regen, str_userid)


def player_death(event_var):
   """Calls player_spawn"""
   player_spawn(event_var)


def player_disconnect(event_var):
   """Calls player_spawn"""
   player_spawn(event_var)


def unload():
   """Cancels outstanding regeneration delays"""
   for str_userid in list_players:
      gamethread.cancelDelayed('healthregen_%s' % str_userid)

   for str_option in dict_options:
      es.set(str_option, 0)


def execute_regen(str_userid):
   """
   Checks to make sure the player still needs health
   Adds the required health to the player and executes another iteration if the player needs health after addition
   """
   global list_players

   int_health = es.getplayerprop(str_userid, 'CBasePlayer.m_iHealth')
   int_health_max = int(es.ServerVar('healthregen_threshold'))
   if int_health_max > int_health:
      int_health += int(es.ServerVar('healthregen_iteration_amount'))
      if int_health_max > int_health:

         int_health_min = int(es.ServerVar('healthregen_start_health'))
         if int_health_min > int_health:
            int_health = int_health_min
         es.setplayerprop(str_userid, 'CBasePlayer.m_iHealth', int_health)
         gamethread.delayedname(es.ServerVar('healthregen_iteration_delay'), 'healthregen_%s' % str_userid, execute_regen, str_userid)

      else:
         es.setplayerprop(str_userid, 'CBasePlayer.m_iHealth', int_health_max)
         list_players.remove(str_userid)